<?php
/**
 * @file
 * Adds contextual information to 'Create content' menu links.
 */

// Load menu module for additional helper hooks
module_load_include('inc', 'create_content', 'includes/menu.create_content');

/**
 * Load all create content includes.
 */
function create_content_load_all_includes($load_all_includes = TRUE) {
  foreach (module_list(FALSE, TRUE) as $module) {
    if (!$load_all_includes && variable_get('create_content_'. $module .'_enabled', '1') != '1') {
      continue;
    }

    module_load_include('create_content.inc', 'create_content', 'includes/'. $module);
  }
}

/**
 * Load enabled create content includes.
 */
function create_content_load_enabled_includes() {
  create_content_load_all_includes(FALSE);
}

/**
 * Collect, alter and store the context definitions.
 */
function create_content_get_contexts() {
  static $contexts;

  if (isset($contexts)) {
    return $contexts;
  }

  create_content_load_enabled_includes();

  // Build context
  $contexts = create_content_build_contexts();

  // Alter context.
  drupal_alter('create_content', $contexts);

  // Set title
  if ($contexts) {
    $router_item = create_content_menu_get_router_item();
    create_content_set_title($router_item['title']);
  }

  // DEBUG:
  // dsm($node); dsm($router_item); dsm($contexts);

  return $contexts;
}

/**
 * Build context object based on the selected $router_item.
 */
function create_content_build_contexts() {
  $contexts = array();
  foreach (module_implements('create_content') as $module) {
    // Check that create content is enabled.
    if (variable_get('create_content_'. $module .'_enabled', '1') != '1') {
      continue;
    }

    $module_contexts = call_user_func($module .'_create_content');
    if (isset($module_contexts) && is_array($module_contexts)) {
      foreach ($module_contexts as $context_name => $module_context) {
        $contexts[$context_name] = $module_context;
        $contexts[$context_name]['name'] = $context_name;
      }
    }
  }

  return $contexts;
}


////////////////////////////////////////////////////////////////////////////////
// Utility functions
////////////////////////////////////////////////////////////////////////////////

/**
 * Filter create content context to apply only one type of context based on a key.
 */
function create_content_filter_context(&$context, $filter_keys) {
  $filter_keys = (is_array($filter_keys)) ? $filter_keys : array($filter_keys);

  $has_filter_context = FALSE;
  foreach ($context as $key => $value) {
    foreach ($filter_keys as $filter_key) {
      if (strpos($key, $filter_key) === 0) {
        $has_filter_context = TRUE;
        break;
      }
    }
  }

  if (!$has_filter_context) {
    return;
  }

  foreach ($context as $key => $value) {
    foreach ($filter_keys as $filter_key) {
      if (strpos($key, $filter_key) === FALSE) {
        unset($context[$key]);
      }
    }
  }
}

/**
 * Get associative array of all CCK fields for selecte type.
 */
function create_content_get_fields($type) {
  $result = db_query("SELECT field_name FROM {content_node_field} WHERE type='%s' ORDER BY field_name", $type);
  $fields = array();
  while ($record = db_fetch_array($result)) {
    $fields[$record['field_name']] = $record['field_name'];
  }
  return $fields;
}

/**
 * Get current context's path.
 */
function create_content_get_path() {
  if (isset($_GET['create-content']) && $_GET['create-content'] != 1) {
    return $_GET['create-content'];
  }
  else {
    return $_GET['q'];
  }
}

/**
 * Build prepopulate query parameter for cck fields.
 */
function create_content_build_field_query($node, $field_name, $parent_name = NULL) {
  $field = $node->$field_name;

  // Build default query
  $param_name = create_content_format_field_query_param($field_name, $parent_name);
  $query = '';
  foreach ($field as $index => $item) {
    $query .= (($query) ? '&' : '') . $param_name .'['. $index .']='. $item['value'];
  }
  return $query;
}

/**
 * Format CCK field query param.
 */
function create_content_format_field_query_param($field_name, $parent_name = NULL) {
  return ($parent_name) ? 'edit['. $parent_name .']['. $field_name .'][value]' : 'edit['. $field_name .'][value]';
}

/**
 * Get current context's router item.
 */
function create_content_menu_get_router_item() {
  return menu_get_item( create_content_get_path() );
}

/**
 * Get current context's node.
 */
function create_content_node_load() {
  if (preg_match('#^node/(\d+)#', create_content_get_path() , $match)) {
    return node_load($match[1]);
  }
  else {
    return NULL;
  }
}

/**
 * Set the current context title.
 */
function create_content_set_title($title = NULL) {
  static $stored_title;

  if (isset($title)) {
    $stored_title = $title;
  }
  return $stored_title;
}

/**
 * Get the current context title.
 */
function create_content_get_title() {
  return create_content_set_title();
}

/**
 * Get current context's views.
 */
function create_content_get_views($view = NULL) {
  static $views = array();

  if (!is_null($view)) {
    $views[ $view->name ] = $view;
  }

  return $views;
}

/**
 * Get current context's views.
 */
function create_content_set_view($view_name, $view_display_id = 'default', $view_args = array()) {
  // Load and execute view.
  $view = views_get_view($view_name);

  if (!$view) {
    return NULL;
  }

  // Remove a feed display since they will never be applicable and can cause
  // duplicate feed links. http://drupal.org/node/997168
  foreach ($view->display as $display_name => $display) {
    if ($display->display_plugin == 'feed') {
      unset($view->display[$display_name]);
    }
  }

  // Set current node so that the views_attach.module can populate the
  // arguments using the node.
  $view->current_node = create_content_node_load();

  // Set display
  $view->set_display($view_display_id);

  // Set args
  if ($view_args) {
    $view->set_arguments($view_args);
  }

  // Build and execute
  $view->build();
  $view->execute();


  // DEBUG: View arguments
  // dsm($view_name .': '. $view_display_id . (($view_args) ? ' ('. implode(',', $view_args) .')' : '') );

  // DEBUG:
  // dsm($view); dsm(view_create_content_get_allowed_types($view));

  // Store a reference to the view
  create_content_get_views($view);

  return $view;
}
