<?php
/**
 * @file
 * Provides utility functions for the content.module (CCK).
 *
 * Individual CCK field modules and Views will implement the needed hooks.
 *
 * See
 * - content_taxonomy.create_content.inc
 * - nodereference.create_content.inc
 * - views.create_content.inc
 */


////////////////////////////////////////////////////////////////////////////////
// Utility functions.
////////////////////////////////////////////////////////////////////////////////

/**
 * Get inherited content field context
 */
function content_create_content_get_inherited_field_context($node, $field_name) {
  if ( !isset($node->$field_name) ) {
    return;
  }

  $values = array();
  foreach ($node->$field_name as $index => $item) {
    $values[] = $item['value'];
  }

  $allowed_types = array();
  content_create_content_get_field_instance_allowed_types($allowed_types, $field_name, $values );
  return array(
    'allowed_types' => $allowed_types,
  );
}

/**
 * Get parent content field context
 */
function content_create_content_get_parent_field_context($node, $field_name, $prefix = '') {
  $field = $node->$field_name;
  if (!$field  || !$field[0] ) {
    return;
  }

  $parent_node = node_load($field[0]['nid']);
  $parent_router_item = menu_get_item('node/'. $parent_node->nid);
  $contexts = create_content_build_contexts($parent_node, $parent_router_item);
  foreach ($contexts as $context_name => $context) {
    unset($context[$context_name]);
    $context[$prefix . $context_name] = $context;
  }

  // Set title
  create_content_set_title($parent_router_item['title']);

  return $contexts;
}

/**
 * Get allowed type for CCK content field instance.
 *
 * This function can be used in hook_create_context() to correct prepopulated
 * CCK fields. Additional field handling could be added for fields that do
 * not store data a [value] field.
 * (ie @link http://drupal.org/project/link link.module @endlink and @link http://drupal.org/project/email email.module @endlink).
 *
 * @param &$allowed_types
 *   An associative array of already defined allowed types.
 *
 * @param $field
 *   The field or field name that is going to be prepopulated.
 *
 * @param $values
 *   An array of values or just a single value that will be used to prepopulate the field.
 *
 * @param $types (optional)
 *   An array of types or just a single types that can be defined in the $allowed_types variables.
 */
function content_create_content_get_field_instance_allowed_types(&$allowed_types, $field, $values, $types = NULL) {
  // Set $allowed_types
  $allowed_types = ($allowed_types) ? $allowed_types : array();

  // Set $field.
  if (is_string($field)) {
    // Load field if it is passed as a string.
    $field = db_fetch_array(db_query("SELECT field_name, type, multiple FROM {content_node_field} WHERE field_name='%s'", $field));
  }

  // Set $values
  $values = (is_array($values)) ? $values : array($values);

  // Set $type
  $types = (is_string($types)) ? array($types) : $types;

  // Build sql query to look up field instance information for the selected field.
  $sql = "SELECT type_name, widget_type, display_settings FROM {content_node_field_instance} WHERE field_name='%s'";
  $params = array($field['field_name']);
  if (!is_null($types) && count($types)) { // Apply type filter
    $sql .= ' AND type_name IN ('. db_placeholders($types, 'text') .')';
    $params = array_merge($params, $types);
  }

  $field_result = db_query($sql, $params);
  while ($field_info = db_fetch_array($field_result)) {

    // Include parent fieldset name when applicable.
    $display_settings = unserialize($field_info['display_settings']);
    if ($parent_name = $display_settings['parent']) {
      $param_name = 'edit['. $parent_name .']['. $field['field_name'] .']';
    }
    else {
      $param_name = 'edit['. $field['field_name'] .']';
    }

    // Build prepopulate query parameters based on the field type.
    // See prepopulate/USAGE.txt for information about formatting.
    $query = '';
    switch ($field['type']) {

      // Using unfolded loop to improve performance.

      // Nodereference and default widget handling for buttons and select is
      // almost identical but it seems smarter to show exactly when is going on.
      case 'nodereference':

        switch ($field_info['widget_type']) {
          case 'nodereference_buttons':
            foreach ($values as $param_index => $param_value) {
              $query .= (($query) ? '&' : '') . $param_name .'[nid][nid]['. $param_value .']='. $param_value;
            }
            break;
          case 'nodereference_select':
            foreach ($values as $param_index => $param_value) {
              $query .= (($query) ? '&' : '') . $param_name .'[nid][nid]='. $param_value;
            }
            break;
          case 'nodereference_autocomplete':
            foreach ($values as $param_index => $param_value) {
              $query .= (($query) ? '&' : '') . $param_name .'['. $param_index .'][nid][nid]=[nid:%20'. $param_value .']';
            }
            break;
          default:
            // TODO: Handle default case.
        }
        break;

      default:

        switch ($field_info['widget_type']) {
          case 'optionwidgets_buttons':
            foreach ($values as $param_index => $param_value) {
              $query .= (($query) ? '&' : '') . $param_name .'[value]['. urlencode($param_value) .']='. urlencode($param_value);
            }
            break;
          case 'optionwidgets_select':
            foreach ($values as $param_index => $param_value) {
              $query .= (($query) ? '&' : '') . $param_name .'[value]='. urlencode($param_value);
            }
            break;
          default:
            foreach ($values as $param_index => $param_value) {
              $query .= (($query) ? '&' : '') . $param_name .'[value]['. $param_index .']='. urlencode($param_value);
            }
            break;
        }
        break;

    }

    // Set or append query to allowed types.
    $type = $field_info['type_name'] ;
    if ($query) {
      if ( empty($allowed_types[$type]) || $allowed_types[$type] == $type ) {
        $allowed_types[$type] = $query;
      }
      else {
        $allowed_types[$type] .= '&'. $query;
      }
    }

  }

  // DEBUG:
  // dsm($allowed_types);
}
